﻿
// =============================================================================== 
// Development Header (C# .NET) 
// =============================================================================== 
// Bombardier 
// Orbiflo  : FleetOverviewCarStatusSummaryControl
// 
// Purpose  : User control to be disp;ayed in fleet overview grid
// 
// =============================================================================== 
// Copyright © Bombardier. All rights reserved. 
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY 
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT 
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
// FITNESS FOR A PARTICULAR PURPOSE. 
// =============================================================================== 
// 
// Modified: 
// By                        Date           Description 
// ------                   --------        ----------- 
// Udaya sree             27-02-2012         Created   
// ===============================================================================

namespace Bombardier.Orbiflo.Orbita.ClientModules.AlertMonitoringModule.FleetOverview.Controls
{
    using System;
    using System.Collections.ObjectModel;
    using System.Globalization;
    using System.Windows.Forms;
    using ClientLogic.PresentationEntities.Alerts.Monitoring;
    using Common.Constants;
    using Common.Entities.Alerts;
    using Common.Utilities;
    using Common.Utilities.Ocs;
    using Interfaces;
    using Windows.Interface.Common;

    /// <summary>
    /// Describes the implementation details of Fleet OverView Car Status Summary Control<see cref="FleetOverviewCarStatusSummaryControl"/> Object.
    /// This is a common control class will utilized in all the Car Cells of Fleet Overview Grid control.
    /// Displays the common layout with the information of Last Event, Critical Event, Event Location and Alerts Awaiting for Acknowledgment/Reactivation. 
    /// </summary>
    public partial class FleetOverviewCarStatusSummaryControl : UserControl, IFleetOverviewCarStatusSummaryControl
    {
        /// <summary>
        ///  Describes the Status Summary of Alerts and Events on Cars in the Consist.
        /// </summary>
        private FleetOverviewCarStatus cachedFleetOverviewCarStatus;

        /// <summary>
        /// Initializes a new instance of the <see cref="FleetOverviewCarStatusSummaryControl"/> class.
        /// </summary>
        public FleetOverviewCarStatusSummaryControl()
        {
            // Exceptions thrown here will have to be caught by the consumer of this User Control.
            this.InitializeComponent();
            this.LocalizeUi();
        }

        /// <summary>
        /// This method will be used to localize the Labels information on the UI.
        /// </summary>
        private void LocalizeUi()
        {
            // This doesn't need localising so far
            // this.UbCarId.Text = "Car Id";
            this.UbLastEventAt.Text = AlertMonitoringFleetOverviewResourceManager.GetString("LastEventAtAbbreviated");
            this.UbAlertsCount.Text = AlertMonitoringFleetOverviewResourceManager.GetString("AlertStatusIndicatorPrefix") + Delimiter.EqualsWithSpaces + 0;
            this.UbLastCriticalEventAt.Text = AlertMonitoringFleetOverviewResourceManager.GetString("LastCriticalEventAtAbbreviated"); // "CE";
        }

        /// <summary>
        /// Gets or sets Cached Fleet Overview Car Status<see cref="cachedFleetOverviewCarStatus"/> Object.
        /// </summary>
        public FleetOverviewCarStatus FleetOverviewCarStatus
        {
            get
            {
                // Store a cached instance of the list.  The reason that we need to store a cached
                // instance of this list is because the grid determines whether an object has changed
                // based on that particular instance, not any sub-objects of that instance.  For example,
                // if the grid value is an integer, the grid knows that the value changes simply by determining
                // if the values are not equal, marking the row as dirty accordingly.  In the case of having 
                // a class instance in the cell (such as we do here), if we kept the same instance of the class
                // around through all the changes, the grid would always see that we still have the same object
                // in the cell, since it doesn't know about the sub objects.  When the grid sees the same objects, 
                // it does not mark the row as dirty and will not fire the various update-related events; furthermore,
                // actions like pressing Esc will not work because the grid does not have a different value to assign
                // back to the cell.  We work around all this behavior by returning a new list object when any of the
                // values have been changed.
                if (this.cachedFleetOverviewCarStatus != null)
                {
                    return this.cachedFleetOverviewCarStatus;
                }

                var fleetOverviewCarStatus = new FleetOverviewCarStatus
                                                 {
                                                     AssetInfo = { Label = this.UbCarId.Text },
                                                     LastEventAt = TimeZoneUtilities.ConvertTimeToUtc(Convert.ToDateTime(this.LastEventAtLabel.Text)),
                                                     LastCriticalEventAt =
                                                         TimeZoneUtilities.ConvertTimeToUtc(Convert.ToDateTime(this.LastCriticalEventAtLabel.Text)),
                                                     LastEventLocation = this.LocationDescLabel.Text,
                                                     LasteventBackColor = this.UbLastEventAt.Appearance.BackColor,
                                                     LasteventForeColor = this.UbLastEventAt.Appearance.ForeColor,
                                                     LastCriticalEventBackColor = this.UbLastCriticalEventAt.Appearance.BackColor,
                                                     LastCriticalEventForeColor = this.UbLastCriticalEventAt.Appearance.ForeColor,
                                                     AlertStatusBackcolor = this.UbAlertsCount.Appearance.BackColor,
                                                     AlertStatusForeColor = this.UbAlertsCount.Appearance.ForeColor,
                                                     IsAlertStatusIndicatorEnabled = this.UbAlertsCount.Enabled,
                                                     IsLastCriticalEventEnabled = this.UbLastCriticalEventAt.Enabled
                                                 };
                if (this.UbAlertsCount.Enabled)
                {
                    {
                        // if (this.UbAlertsCount.Appearance.BackColor == Color.Red)
                        if (Convert.ToBoolean(this.UbAlertsCount.Tag))
                        {
                            fleetOverviewCarStatus.AlertsAwaitingAcknolodgementCount = Convert.ToInt64(this.UbAlertsCount.Text);
                        }
                        else
                        {
                            fleetOverviewCarStatus.AlertsAwaitingReActivationCount = Convert.ToInt64(this.UbAlertsCount.Text);
                        }
                    }
                }
                else
                {
                    fleetOverviewCarStatus.AlertsAwaitingAcknolodgementCount = 0;
                    fleetOverviewCarStatus.AlertsAwaitingReActivationCount = 0;
                }

                this.cachedFleetOverviewCarStatus = fleetOverviewCarStatus;
                return this.cachedFleetOverviewCarStatus;
            }

            set
            {
                if (this.cachedFleetOverviewCarStatus == value)
                {
                    return;
                }

                this.cachedFleetOverviewCarStatus = value;
                this.UpdateUi();
            }
        }

        #region Methods

        /// <summary>
        /// This method will be used to update the Formats the UI and updates information for individual cells(Cars) of Fleet Overview Grid by updating the user control.
        /// </summary>
        private void UpdateUi()
        {
            if (this.cachedFleetOverviewCarStatus == null)
            {
                this.ResetCarControl();
            }

            var fleetOverviewCarStatus = this.FleetOverviewCarStatus;
            this.UbCarId.Text = fleetOverviewCarStatus.AssetInfo.Label;
            if (fleetOverviewCarStatus.LastEventAt == default(DateTime).ToDisplayTZ())
            {
                this.LastEventAtLabel.Text = string.Empty;
                this.UbLastEventAt.Enabled = false;
                this.UbCarId.Enabled = false;
            }
            else
            {
                // display time should be consistent with EDS - Issue 6834 fix
                this.LastEventAtLabel.Text = fleetOverviewCarStatus.LastEventAt.ToCurrentCultureTimePrecision();
                this.UbLastEventAt.Enabled = true;
                this.UbCarId.Enabled = true;
            }

            // display time should be consistent with EDS - Issue 6834 fix
            this.LastCriticalEventAtLabel.Text = fleetOverviewCarStatus.LastCriticalEventAt == default(DateTime).ToDisplayTZ() ? string.Empty : fleetOverviewCarStatus.LastCriticalEventAt.ToCurrentCultureTimePrecision();
            if (fleetOverviewCarStatus.AlertsAwaitingAcknolodgementCount > 0)
            {
                this.UbAlertsCount.Text = AlertMonitoringFleetOverviewResourceManager.GetString("AlertStatusIndicatorPrefix") + Delimiter.EqualsWithSpaces + fleetOverviewCarStatus.AlertsAwaitingAcknolodgementCount;
            }
            else if (fleetOverviewCarStatus.AlertsAwaitingReActivationCount > 0)
            {
                this.UbAlertsCount.Text = AlertMonitoringFleetOverviewResourceManager.GetString("AlertStatusIndicatorPrefix") + Delimiter.EqualsWithSpaces + fleetOverviewCarStatus.AlertsAwaitingReActivationCount;
            }
            else
            {
                this.UbAlertsCount.Text = AlertMonitoringFleetOverviewResourceManager.GetString("AlertStatusIndicatorPrefix") + Delimiter.EqualsWithSpaces + 0;
            }

            this.UbAlertsCount.Tag = fleetOverviewCarStatus.HasTriggeredConsistAlert;
            this.LocationDescLabel.Text = fleetOverviewCarStatus.LastEventLocation;
            this.LocationDescLabel.Enabled = !this.LocationDescLabel.Text.Equals(AlertMonitoringFleetOverviewResourceManager.GetString("NoLocationAvailabale"));

            this.UbLastEventAt.Appearance.BackColor = fleetOverviewCarStatus.LasteventBackColor;
            this.UbLastEventAt.Appearance.ForeColor = fleetOverviewCarStatus.LasteventForeColor;
            this.UbLastCriticalEventAt.Appearance.BackColor = fleetOverviewCarStatus.LastCriticalEventBackColor;
            this.UbLastCriticalEventAt.Appearance.ForeColor = fleetOverviewCarStatus.LastCriticalEventForeColor;
            this.UbAlertsCount.Appearance.BackColor = fleetOverviewCarStatus.AlertStatusBackcolor;
            this.UbAlertsCount.Appearance.ForeColor = fleetOverviewCarStatus.AlertStatusForeColor;
            this.UbAlertsCount.Enabled = fleetOverviewCarStatus.IsAlertStatusIndicatorEnabled;
            this.UbLastCriticalEventAt.Enabled = fleetOverviewCarStatus.IsLastCriticalEventEnabled;
            if (this.UbAlertsCount.Enabled)
            {
                this.UbAlertsCount.Cursor = Cursors.Hand;
            }

            this.UbCarId.Cursor = Cursors.Hand;
            if (this.UbLastEventAt.Enabled)
            {
                this.UbLastEventAt.Cursor = Cursors.Hand;
            }

            if (this.UbLastCriticalEventAt.Enabled)
            {
                this.UbLastCriticalEventAt.Cursor = Cursors.Hand;
            }
        }

        /// <summary>
        /// This method will be used to Resets the UI Format and information for individual cells(Cars) of Fleet Overview Grid.
        /// </summary>
        private void ResetCarControl()
        {
            this.UbCarId.Text = null;
            this.LastEventAtLabel.Text = null;
            this.LastCriticalEventAtLabel.Text = null;
            this.UbAlertsCount.Text = null;
            this.LocationDescLabel.Text = null;
        }

        #endregion

        #region button click events

        /// <summary>
        /// Event that fires when Car description is clicked.
        /// </summary>
        /// <param name="sender"> Sender of the event. </param>
        /// <param name="e"> EventArgs variable. </param>
        private void UbCarIdClick(object sender, EventArgs e)
        {
            try
            {
                Cursor.Current = Cursors.WaitCursor;
                this.SwitchToEventDurationSummary(sender, this.UbCarId.Text);
            }
            catch (Exception ex)
            {
                ExceptionManager.HandleException(
                    AlertMonitoringFleetOverviewResourceManager.GetString("EDSSwitchError"),
                    AlertMonitoringFleetOverviewResourceManager.GetString("ModuleName"),
                    ex,
                    TraceSeverity.Error,
                    TraceCategory.Framework,
                    MessageAction.ShowAndWrite);
            }
            finally
            {
                Cursor.Current = Cursors.Default;    
            }
        }

        /// <summary>
        /// Event that fires when last event at is clicked.
        /// </summary>
        /// <param name="sender"> Sender of the event. </param>
        /// <param name="e"> EventArgs variable. </param>
        private void UbLastEventAtClick(object sender, EventArgs e)
        {
            try
            {
                Cursor.Current = Cursors.WaitCursor;
                this.SwitchToEventDurationSummary(sender, this.UbLastEventAt.Text);
            }
            catch (Exception ex)
            {
                ExceptionManager.HandleException(
                    AlertMonitoringFleetOverviewResourceManager.GetString("EDSSwitchError"),
                    AlertMonitoringFleetOverviewResourceManager.GetString("ModuleName"),
                    ex,
                    TraceSeverity.Error,
                    TraceCategory.Framework,
                    MessageAction.ShowAndWrite);
            }
            finally
            {
                Cursor.Current = Cursors.Default;
            }
        }

        /// <summary>
        /// Event that fires when last critical event at is clicked.
        /// </summary>
        /// <param name="sender"> Sender of the event. </param>
        /// <param name="e"> EventArgs variable. </param>
        private void UbLastCriticalEventAtClick(object sender, EventArgs e)
        {
            try
            {
                Cursor.Current = Cursors.WaitCursor;
                this.SwitchToEventDurationSummary(sender, this.UbLastCriticalEventAt.Text);
            }
            catch (Exception ex)
            {
                ExceptionManager.HandleException(
                    AlertMonitoringFleetOverviewResourceManager.GetString("EDSSwitchError"),
                    AlertMonitoringFleetOverviewResourceManager.GetString("ModuleName"),
                    ex,
                    TraceSeverity.Error,
                    TraceCategory.Framework,
                    MessageAction.ShowAndWrite);
            }
            finally
            {
                Cursor.Current = Cursors.Default;
            }
        }

        /// <summary>
        /// Event that fires when alert status indicator is clicked.
        /// </summary>
        /// <param name="sender"> Sender of the event. </param>
        /// <param name="e"> EventArgs variable. </param>
        private void UbAlertsCountClick(object sender, EventArgs e)
        {
            try
            {
                Cursor.Current = Cursors.WaitCursor;
                this.RedirectToAlertDetailBrowser(sender);
            }
            catch (Exception ex)
            {
                ExceptionManager.HandleException(
                    AlertMonitoringFleetOverviewResourceManager.GetString("ADBSwitchError"),
                    AlertMonitoringFleetOverviewResourceManager.GetString("ModuleName"),
                    ex,
                    TraceSeverity.Error,
                    TraceCategory.Framework,
                    MessageAction.ShowAndWrite);
            }
            finally
            {
                Cursor.Current = Cursors.Default;
            }
        }

        /// <summary>
        /// Event that fires when location description is clicked.
        /// </summary>
        /// <param name="sender"> Sender of the event. </param>
        /// <param name="e"> EventArgs variable. </param>
        private void LocationDescLabelClick(object sender, EventArgs e)
        {
            try
            {
                string assetLabel = this.UbCarId.Text;
                if (this.LocationSelected != null)
                {
                    this.LocationSelected(sender, new EventArgs<string>(assetLabel));
                }
            }
            catch (Exception ex)
            {
                ExceptionManager.HandleException(
                    AlertMonitoringFleetOverviewResourceManager.GetString("MapLocationDisplayError"),
                    AlertMonitoringFleetOverviewResourceManager.GetString("ModuleName"),
                    ex,
                    TraceSeverity.Error,
                    TraceCategory.Framework,
                    MessageAction.ShowAndWrite);
            }
        }

        /// <summary>
        /// This method will be used to Loads the Event duration summary from Alert Monitoring module and displays the events.
        /// </summary>
        /// <param name="sender"> Sender of the event. </param>
        /// <param name="btnText"> String to read the text of button. </param>
        private void SwitchToEventDurationSummary(object sender, string btnText)
        {
            var carDescription = this.UbCarId.Text;
            var asset = new Asset { Label = carDescription };
            var fleetOverviewCarStatus = new FleetOverviewCarStatus { AssetInfo = asset };
            if (this.CarDescriptionClicked != null)
            {
                this.CarDescriptionClicked(sender, new EventArgs<FleetOverviewCarStatus, string>(fleetOverviewCarStatus, btnText));
            }
        }

        /// <summary>
        /// This method will be used to redirecting to Alert Detail Browser view.
        /// </summary>
        /// <param name="sender"> Sender of the event. </param>
        private void RedirectToAlertDetailBrowser(object sender)
        {
            var filterCriteriaList = new Collection<AlertDetailFilterCriteria>();
            var filterCriteriaCar = new AlertDetailFilterCriteria
                                        {
                                            FilterLevel = AlertDetailFilterLevel.Specific,
                                            FilterName = this.UbCarId.Text,
                                            FilterType = AlertDetailFilterType.Car
                                        };

            filterCriteriaList.Add(filterCriteriaCar);
            if (this.AlertStatusIndicatorClicked != null)
            {
                this.AlertStatusIndicatorClicked(sender, new EventArgs<Collection<AlertDetailFilterCriteria>>(filterCriteriaList));
            }
        }

        #endregion button click events

        #region IFleetOverviewCarStatusSummaryControl members

        /// <summary>
        /// Event handler to load the Event Duration Summary module when clicked on the Car Description(Last Event) Button.
        /// Fleet Overview Car Status Object and Car Label are passing as Event Arguments.
        /// </summary>
        public event EventHandler<EventArgs<FleetOverviewCarStatus, string>> CarDescriptionClicked;

        /// <summary>
        /// Event handler to load/update the Alert Detail Browser view when clicked on the Alert Status Indicator Button.
        /// </summary>
        public event EventHandler<EventArgs<Collection<AlertDetailFilterCriteria>>> AlertStatusIndicatorClicked;

        /// <summary>
        /// Event handler load/update the Map view when clicked on the Location link.
        /// Asset Label is passing as Event Argument.
        /// </summary>
        public event EventHandler<EventArgs<string>> LocationSelected;

        #endregion IFleetOverviewCarStatusSummaryControl members
    }
}
