﻿
// =============================================================================== 
// Development Header (C# .NET) 
// =============================================================================== 
// Bombardier 
// Orbiflo  : FleetOverView
// 
// Purpose  : Contains controls of Fleet overview to be displayed
// 
// =============================================================================== 
// Copyright © Bombardier. All rights reserved. 
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY 
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT 
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
// FITNESS FOR A PARTICULAR PURPOSE. 
// =============================================================================== 
// 
// Modified: 
// By                        Date           Description 
// ------                   --------        ----------- 
// Udaya sree             27-02-2012         Created   
// UdayaSree              28-05-2012         Removed alertId from UpdateAlertState parameters as it is not used.
// UdayaSree              01-06-2012         Removed UpdateAlertState as implementation moved to presenter.
// Sailaja                02-07-2012         Fixed the issue with maximum cars to be displayed. If it is not
//                                           available in the db configuration, it is showing only one car.
// UdayaSree              25-01-2013         added setRowHeight field  - Defect 9566 fix
// ===============================================================================

namespace Bombardier.Orbiflo.Orbita.ClientModules.AlertMonitoringModule.FleetOverview.Views
{
    using System;
    using System.Collections.ObjectModel;
    using System.Linq;
    using System.Windows.Forms;
    using ClientLogic.PresentationEntities.Alerts.Monitoring;
    using Common.Utilities.Ocs;
    using Infragistics.Win;
    using Infragistics.Win.UltraWinDataSource;
    using Infragistics.Win.UltraWinGrid;
    using Interfaces;
    using Windows.Interface.Common;

    /// <summary>
    /// Describes the implementation details of Fleet Overview Object.
    /// </summary>
    public partial class FleetOverView : UserControl, IFleetOverView
    {
        /// <summary>
        /// Collection of Fleet overview consist status <see cref="FleetOverviewConsistStatus"/> Objects.
        /// Status Summary of Alerts on the Consist and Alerts and Events on Cars in the Consist.
        /// </summary>
        private Collection<FleetOverviewConsistStatus> fleetOverviewConsistStatuses;

        /// <summary>
        /// Maximum cars to be displayed for a consist.
        /// </summary>
        private int maxCarsTobeDisplayed;

        /// <summary>
        /// Consist Group Header for defining the Header properties of Ultra Grid columns of Consist Cells.
        /// </summary>
        private UltraGridGroup consistGroupHeader;

        /// <summary>
        /// Car Group Header for defining the Header properties of Ultra Grid columns of Car Cells(i.e Cars Comprising Consist).
        /// </summary>
        private UltraGridGroup carsGroupHeader;

        /// <summary>
        /// Indicates whether row hieght is set initially or not.
        /// </summary>
        private bool setRowHeight;

        #region Constructor
        /// <summary>
        /// Initializes a new instance of the <see cref="FleetOverView"/> class.
        /// </summary>
        public FleetOverView()
        {
            try
            {
                this.InitializeComponent();

                // Rows cannot be deleted - issue 6769 fix 
                this.ugFleetOverViewStatusSummary.DisplayLayout.Override.AllowDelete = DefaultableBoolean.False;

                // Resizing of columns should not be allowed  - issue 6772 fix
                this.ugFleetOverViewStatusSummary.DisplayLayout.Override.AllowColSizing = AllowColSizing.None;

                // Selection of multiple cells and clicking on one selected cell is making the selected cell blank - Issue 6774 fix
                this.ugFleetOverViewStatusSummary.DisplayLayout.Override.AllowMultiCellOperations = AllowMultiCellOperation.None;
                this.UcFleetOverviewCarStatusSummaryControl.CarDescriptionClicked += this.UcFleetOverviewCarStatusSummaryControlCarDescriptionClicked;
                this.UcFleetOverviewCarStatusSummaryControl.AlertStatusIndicatorClicked += this.UcFleetOverviewCarStatusSummaryControlAlertStatusIndicatorClicked;
                this.UcFleetOverviewCarStatusSummaryControl.LocationSelected += this.UcFleetOverviewCarStatusSummaryControlLocationSelected;
                setRowHeight = true;
            }
            catch (Exception ex)
            {
                ExceptionManager.HandleException(
                    AlertMonitoringFleetOverviewResourceManager.GetString("FleetOverviewInitializeError"),
                    AlertMonitoringFleetOverviewResourceManager.GetString("ModuleName"),
                    ex,
                    TraceSeverity.Error,
                    TraceCategory.Framework,
                    MessageAction.ShowAndWrite);
            }
        }

        #endregion Constructor

        #region user control click events

        /// <summary>
        /// Click event for alert status Indicator clicked.
        /// </summary>
        /// <param name="sender"> Sender of the event. </param>
        /// <param name="e"> Event Arguments variable. </param>
        public void UcFleetOverviewCarStatusSummaryControlAlertStatusIndicatorClicked(object sender, EventArgs<Collection<AlertDetailFilterCriteria>> e)
        {
            try
            {
                if (this.AlertStatusIndicatorClicked != null)
                {
                    this.AlertStatusIndicatorClicked(sender, e);
                }
            }
            catch (Exception ex)
            {
                ExceptionManager.HandleException(
                    AlertMonitoringFleetOverviewResourceManager.GetString("AlertDetailsDisplayError"),
                    AlertMonitoringFleetOverviewResourceManager.GetString("ModuleName"),
                    ex,
                    TraceSeverity.Error,
                    TraceCategory.Framework,
                    MessageAction.ShowAndWrite);
            }
        }

        /// <summary>
        ///  Click event for car description clicked.
        /// </summary>
        /// <param name="sender"> Sender of the event. </param>
        /// <param name="e"> Event Arguments variable. </param>
        public void UcFleetOverviewCarStatusSummaryControlCarDescriptionClicked(object sender, EventArgs<FleetOverviewCarStatus, string> e)
        {
            try
            {
                if (this.CarDescriptionClicked != null)
                {
                    this.CarDescriptionClicked(sender, e);
                }
            }
            catch (Exception ex)
            {
                ExceptionManager.HandleException(
                    AlertMonitoringFleetOverviewResourceManager.GetString("LatestEventOnCarDisplayError"),
                    AlertMonitoringFleetOverviewResourceManager.GetString("ModuleName"),
                    ex,
                    TraceSeverity.Error,
                    TraceCategory.Framework,
                    MessageAction.ShowAndWrite);
            }
        }

        /// <summary>
        ///  Click event for location clicked.
        /// </summary>
        /// <param name="sender"> Sender of the event. </param>
        /// <param name="e"> Event Arguments variable. </param>
        public void UcFleetOverviewCarStatusSummaryControlLocationSelected(object sender, EventArgs<string> e)
        {
            try
            {
                if (this.LocationSelected != null)
                {
                    this.LocationSelected(sender, e);
                }
            }
            catch (Exception ex)
            {
                ExceptionManager.HandleException(
                    AlertMonitoringFleetOverviewResourceManager.GetString("MapLocationDisplayError"),
                    AlertMonitoringFleetOverviewResourceManager.GetString("ModuleName"),
                    ex,
                    TraceSeverity.Error,
                    TraceCategory.Framework,
                    MessageAction.ShowAndWrite);
            }
        }

        #endregion user control click events

        #region IFleetOverView methods

        /// <summary>
        /// This method will be used to load Fleet overview and binding the Fleet Overview Data from the Presenter class object.
        /// </summary>
        /// <param name="fleetOverviewConsistStatuses"> Fleet Overview Consist Statuses. </param>
        /// <param name="maxCarsToBeDisplayed"> Maximum Cars to be displayed. </param>
        public void LoadFleetOverview(Collection<FleetOverviewConsistStatus> fleetOverviewConsistStatuses, int maxCarsToBeDisplayed)
        {
            if (InvokeRequired)
            {
                this.Invoke((MethodInvoker)delegate
                {
                    this.LoadFleetOverview(
                        fleetOverviewConsistStatuses,
                        maxCarsToBeDisplayed);
                });
                return;
            }

            try
            {
                this.fleetOverviewConsistStatuses = fleetOverviewConsistStatuses;

                this.maxCarsTobeDisplayed = maxCarsToBeDisplayed;

                this.BindDataSource(maxCarsToBeDisplayed);
            }
            catch (Exception ex)
            {
                ExceptionManager.HandleException(
                    AlertMonitoringFleetOverviewResourceManager.GetString("FleetOverviewInitializeError"),
                    AlertMonitoringFleetOverviewResourceManager.GetString("ModuleName"),
                    ex,
                    TraceSeverity.Error,
                    TraceCategory.Framework,
                    MessageAction.ShowAndWrite);
            }
        }

        /// <summary>
        /// This method will be used to bind the data source of Fleet overview Grid Control and load the Fleet Overview Data.
        /// </summary>
        /// <param name="maxCarsToBeDisplayed"> Maximum Cars to be displayed. </param>
        private void BindDataSource(int maxCarsToBeDisplayed)
        {
            this.consistSummaryDataSource.Rows.Clear();
            this.consistSummaryDataSource.Band.Columns.Clear();
            var maxColumncount = this.fleetOverviewConsistStatuses.AsEnumerable().Select(n => n).Max(x => x.CarsContext.Count);

            // if maximum cars to be displayed is not available in the db configuration, take max cars available in the data.
            if (this.maxCarsTobeDisplayed <= 0)
            {
                this.maxCarsTobeDisplayed = maxColumncount;
            }

            if (maxCarsToBeDisplayed > 0 && maxColumncount > maxCarsToBeDisplayed)
            {
                maxColumncount = maxCarsToBeDisplayed;
            }

            var maxRowCount = this.fleetOverviewConsistStatuses.Count;
            this.consistSummaryDataSource.Band.Key = "Band0"; // AlertMonitoringFleetOverviewResourceManager.GetString("ConsistSummaryDataSourceBand");
            this.consistSummaryDataSource.Band.Columns.Add("Consist", typeof(string));
            for (int i = 0; i < maxColumncount; i++)
            {
                this.consistSummaryDataSource.Band.Columns.Add("Car" + i.ToString(), typeof(FleetOverviewCarStatus));
            }

            this.consistSummaryDataSource.Rows.SetCount(maxRowCount);
            int rowIndex = 0;
            foreach (var consistSummary in this.fleetOverviewConsistStatuses)
            {
                UltraDataRow row = this.consistSummaryDataSource.Rows[rowIndex];
                row[0] = consistSummary.AssetInfo.Label;
                int cellIndex = 1;
                foreach (var car in consistSummary.CarsContext)
                {
                    row[cellIndex] = car;
                    cellIndex++;
                }

                rowIndex++;
            }

            this.ugFleetOverViewStatusSummary.SetDataBinding(this.consistSummaryDataSource, "Band0");
        }

        #endregion IFleetOverView methods

        #region  IFleetOverView events

        /// <summary>
        /// Event to be fired when location label is clicked.
        /// Asset Label is passing as Event Argument.
        /// </summary>
        public event EventHandler<EventArgs<string>> LocationSelected;

        /// <summary>
        /// Event to be fired when car description is clicked.
        /// Fleet Overview Car Status Object and Car Label are passing as Event Arguments.
        /// </summary>
        public event EventHandler<EventArgs<FleetOverviewCarStatus, string>> CarDescriptionClicked;

        /// <summary>
        /// Event to be fired when Alert status indicator is clicked.
        /// </summary>
        public event EventHandler<EventArgs<Collection<AlertDetailFilterCriteria>>> AlertStatusIndicatorClicked;

        #endregion

        #region grid events

        /// <summary>
        /// Event to Initialize grid layout.
        /// </summary>
        /// <param name="sender">  Sender of event. </param>
        /// <param name="e"> Initialize Layout Event Arguments variable .</param>
        private void UgFleetOverViewStatusSummaryInitializeLayout(object sender, InitializeLayoutEventArgs e)
        {
            try
            {
                // Set Row height initially only once as for every Refresh, this method is called - Defect 9566 fix
                if (setRowHeight)
                    this.ugFleetOverViewStatusSummary.DisplayLayout.Override.DefaultRowHeight = this.fleetOverviewCarStatusSummaryControl1.Height + 10;
                setRowHeight = false;
                int targetWidth = Screen.PrimaryScreen.Bounds.Width / (this.maxCarsTobeDisplayed + 1);
                e.Layout.AutoFitStyle = AutoFitStyle.None;
                e.Layout.Scrollbars = Scrollbars.Vertical;
                e.Layout.CaptionVisible = DefaultableBoolean.False;
                this.ugFleetOverViewStatusSummary.SyncWithCurrencyManager = false;

                // ugFleetOverViewStatusSummary.DisplayLayout.Override.SelectedRowAppearance.BackColor = Color.White;
                this.ugFleetOverViewStatusSummary.DisplayLayout.Override.ResetActiveRowAppearance();
                this.ugFleetOverViewStatusSummary.DisplayLayout.Override.ResetActiveCellAppearance();
                this.ugFleetOverViewStatusSummary.DisplayLayout.Override.ActiveAppearancesEnabled = DefaultableBoolean.False;
                e.Layout.Bands[0].RowLayoutStyle = RowLayoutStyle.GroupLayout;

                if (!e.Layout.Bands[0].Groups.Exists("Consist"))
                {
                    this.consistGroupHeader = e.Layout.Bands[0].Groups.Add("Consist");
                    this.consistGroupHeader.Header.Caption = AlertMonitoringFleetOverviewResourceManager.GetString("ConsistColumnCaption");
                }

                if (!e.Layout.Bands[0].Groups.Exists("CarsGroupHeader"))
                {
                    this.carsGroupHeader = e.Layout.Bands[0].Groups.Add("CarsGroupHeader");
                    this.carsGroupHeader.Header.Caption = AlertMonitoringFleetOverviewResourceManager.GetString("CarsGroupHeaderCaption");
                }

                e.Layout.Bands[0].GroupHeadersVisible = true;
                if (this.consistGroupHeader != null)
                {
                    this.consistGroupHeader.Header.Appearance.FontData.Bold = DefaultableBoolean.True;
                    this.consistGroupHeader.Header.Appearance.TextVAlign = VAlign.Middle;
                    this.consistGroupHeader.Header.Appearance.TextHAlign = HAlign.Center;
                }

                if (this.carsGroupHeader != null)
                {
                    this.carsGroupHeader.Header.Appearance.FontData.Bold = DefaultableBoolean.True;
                    this.carsGroupHeader.Header.Appearance.TextVAlign = VAlign.Middle;
                    this.carsGroupHeader.Header.Appearance.TextHAlign = HAlign.Center;

                    foreach (var column in this.ugFleetOverViewStatusSummary.DisplayLayout.Bands[0].Columns)
                    {
                        if (column.Index == 0)
                        {
                            e.Layout.Bands[0].Columns[column.Index].RowLayoutColumnInfo.ParentGroup = this.consistGroupHeader;
                            e.Layout.Bands[0].Columns[column.Index].CellAppearance.TextVAlign = VAlign.Middle;
                            e.Layout.Bands[0].Columns[column.Index].CellAppearance.TextHAlign = HAlign.Center;
                            e.Layout.Bands[0].Columns[column.Index].CellAppearance.FontData.Bold = DefaultableBoolean.True;

                            // Consist name should not be editable - issue 6767 fix.
                            e.Layout.Bands[0].Columns[column.Index].CellActivation = Activation.NoEdit;
                            continue;
                        }

                        e.Layout.Bands[0].Columns[column.Index].Width = targetWidth;
                        e.Layout.Bands[0].Columns[column.Index].RowLayoutColumnInfo.ParentGroup = this.carsGroupHeader;
                    }
                }

                e.Layout.Bands[0].ColHeadersVisible = false;
            }
            catch (Exception ex)
            {
                ExceptionManager.HandleException(
                    AlertMonitoringFleetOverviewResourceManager.GetString("FleetOverviewInitializeError"),
                    AlertMonitoringFleetOverviewResourceManager.GetString("ModuleName"),
                    ex,
                    TraceSeverity.Error,
                    TraceCategory.Framework,
                    MessageAction.ShowAndWrite);
            }
        }

        /// <summary>
        /// Event that initializes a row in grid.
        /// </summary>
        /// <param name="sender">  Sender of event. </param>
        /// <param name="e"> Initialize Row EventArgs variable.</param>
        private void UgFleetOverViewStatusSummaryInitializeRow(object sender, InitializeRowEventArgs e)
        {
            try
            {
                int cellNumber = 0;
                e.Row.Cells[cellNumber].Appearance.BackColor = this.fleetOverviewConsistStatuses[e.Row.Index].BackColor;
                if (this.fleetOverviewConsistStatuses[e.Row.Index].AlertsAwaitingAcknolodgementCount > 0 || this.fleetOverviewConsistStatuses[e.Row.Index].AlertsAwaitingReActivationCount > 0)
                {
                    e.Row.Cells[cellNumber].Appearance.Cursor = Cursors.Hand;
                }

                cellNumber++;
                foreach (var car in this.fleetOverviewConsistStatuses[e.Row.Index].CarsContext)
                {
                    if (cellNumber <= this.fleetOverviewConsistStatuses[e.Row.Index].CarsContext.Count)
                    {
                        e.Row.Cells[cellNumber].Tag = car;
                        e.Row.Cells[cellNumber].EditorComponent = this.foUltraControlContainerEditor;
                        cellNumber++;
                    }
                }

                foreach (var cell in e.Row.Cells)
                {
                    if (cell.Tag != null && cell.Column.DataType == typeof(FleetOverviewCarStatus))
                    {
                        var cellValue = (FleetOverviewCarStatus)cell.Tag;
                        var fleetOverviewCarStatus = new FleetOverviewCarStatus
                                                         {
                                                             AlertScope = cellValue.AlertScope,
                                                             AssetInfo = cellValue.AssetInfo,
                                                             LastCriticalEventAt = cellValue.LastCriticalEventAt,
                                                             LastEventAt = cellValue.LastEventAt,
                                                             LastEventLocation = cellValue.LastEventLocation,
                                                             LasteventForeColor = cellValue.LasteventForeColor,
                                                             LasteventBackColor = cellValue.LasteventBackColor
                                                         };
                        fleetOverviewCarStatus.LasteventForeColor = cellValue.LasteventForeColor;
                        fleetOverviewCarStatus.LastCriticalEventBackColor = cellValue.LastCriticalEventBackColor;
                        fleetOverviewCarStatus.LastCriticalEventForeColor = cellValue.LastCriticalEventForeColor;
                        fleetOverviewCarStatus.AlertStatusBackcolor = cellValue.AlertStatusBackcolor;
                        fleetOverviewCarStatus.AlertStatusForeColor = cellValue.AlertStatusForeColor;
                        fleetOverviewCarStatus.IsAlertStatusIndicatorEnabled = cellValue.IsAlertStatusIndicatorEnabled;
                        fleetOverviewCarStatus.IsLastCriticalEventEnabled = cellValue.IsLastCriticalEventEnabled;
                        fleetOverviewCarStatus.AlertsAwaitingAcknolodgementCount = cellValue.AlertsAwaitingAcknolodgementCount;
                        fleetOverviewCarStatus.AlertsAwaitingReActivationCount = cellValue.AlertsAwaitingReActivationCount;
                        cell.Value = fleetOverviewCarStatus;
                    }
                    else if (cell.Tag == null && cell.Column.Index > 0)
                    {
                        cell.Hidden = true;
                        cell.IsInEditMode = false;
                    }
                }
            }
            catch (Exception ex)
            {
                ExceptionManager.HandleException(
                    AlertMonitoringFleetOverviewResourceManager.GetString("FleetOverviewInitializeError"),
                    AlertMonitoringFleetOverviewResourceManager.GetString("ModuleName"),
                    ex,
                    TraceSeverity.Error,
                    TraceCategory.Framework,
                    MessageAction.ShowAndWrite);
            }
        }

        /// <summary>
        /// Event fired once updation of cell is done in grid.
        /// </summary>
        /// <param name="sender"> Sender of event. </param>
        /// <param name="e"> Cell Event Args variable. </param>
        private void UgFleetOverViewStatusSummaryAfterCellUpdate(object sender, CellEventArgs e)
        {
            try
            {
                if (e.Cell.Column.DataType == typeof(FleetOverviewCarStatus))
                {
                    var fleetOverviewCarStatus = e.Cell.Value as FleetOverviewCarStatus;
                    if (fleetOverviewCarStatus != null)
                    {
                        FleetOverviewCarStatus fleetOverviewCar = fleetOverviewCarStatus;
                        e.Cell.Tag = fleetOverviewCar;
                    }
                }
            }
            catch (Exception ex)
            {
                ExceptionManager.HandleException(
                    AlertMonitoringFleetOverviewResourceManager.GetString("FleetOverviewInfoUpdateError"),
                    AlertMonitoringFleetOverviewResourceManager.GetString("ModuleName"),
                    ex,
                    TraceSeverity.Error,
                    TraceCategory.Framework,
                    MessageAction.ShowAndWrite);
            }
        }

        /// <summary>
        /// Event fired when a cell is clicked in grid.
        /// </summary>
        /// <param name="sender"> Sender of event. </param>
        /// <param name="e"> Click Cell EventArgs variable.</param>
        private void UgFleetOverViewStatusSummaryClickCell(object sender, ClickCellEventArgs e)
        {
            try
            {
                if (e.Cell.Column.Index == 0)
                {
                    if (e.Cell.Appearance.Cursor == Cursors.Hand)
                    {
                        var filterCriteriaList = new Collection<AlertDetailFilterCriteria>();
                        var filterCriteriaCar = new AlertDetailFilterCriteria
                                                    {
                                                        FilterLevel = AlertDetailFilterLevel.Specific,
                                                        FilterName = e.Cell.Value.ToString(),
                                                        FilterType = AlertDetailFilterType.Consist
                                                    };

                        filterCriteriaList.Add(filterCriteriaCar);

                        // if (UcFleetOverviewCarStatusSummaryControlAlertStatusIndicatorClicked != null)
                        this.UcFleetOverviewCarStatusSummaryControlAlertStatusIndicatorClicked(
                            sender, new EventArgs<Collection<AlertDetailFilterCriteria>>(filterCriteriaList));
                    }
                }
            }
            catch (Exception ex)
            {
                ExceptionManager.HandleException(
                    AlertMonitoringFleetOverviewResourceManager.GetString("AlertDetailsDisplayError"),
                    AlertMonitoringFleetOverviewResourceManager.GetString("ModuleName"),
                    ex,
                    TraceSeverity.Error,
                    TraceCategory.Framework,
                    MessageAction.ShowAndWrite);
            }
        }
        #endregion grid events
    }
}
